@@ -102,7 +102,6 @@ static	GScannerConfig	g_scanner_config_template =
 
 
 /* --- prototypes --- */
-extern char* g_vsprintf (gchar *fmt, va_list *args, va_list *args2);
 static inline
 gint		g_scanner_char_2_num	  (guchar	 c,
 					   guchar	 base);
@@ -115,16 +114,16 @@ static gint	g_scanner_key_equal	  (gconstpointer v1,
 static guint	g_scanner_key_hash	  (gconstpointer v);
 static void	g_scanner_get_token_ll	  (GScanner	*scanner,
 					   GTokenType	*token_p,
-					   GValue	*value_p,
+					   GTokenValue	*value_p,
 					   guint	*line_p,
 					   guint	*position_p);
 static void	g_scanner_get_token_i	  (GScanner	*scanner,
 					   GTokenType	*token_p,
-					   GValue	*value_p,
+					   GTokenValue	*value_p,
 					   guint	*line_p,
 					   guint	*position_p);
 static void	g_scanner_free_value	  (GTokenType	*token_p,
-					   GValue	*value_p);
+					   GTokenValue	*value_p);
 
 static guchar	g_scanner_peek_next_char  (GScanner	*scanner);
 static guchar	g_scanner_get_char	  (GScanner	*scanner,
@@ -271,16 +270,12 @@ g_scanner_error (GScanner	*scanner,
   
   if (scanner->msg_handler)
     {
-      va_list args, args2;
+      va_list args;
       gchar *string;
       
       va_start (args, format);
-      va_start (args2, format);
-      string = g_vsprintf ((gchar*) format, &args, &args2);
+      string = g_strdup_vprintf (format, args);
       va_end (args);
-      va_end (args2);
-      
-      string = g_strdup (string);
       
       scanner->msg_handler (scanner, string, TRUE);
       
@@ -298,16 +293,12 @@ g_scanner_warn (GScanner       *scanner,
   
   if (scanner->msg_handler)
     {
-      va_list args, args2;
+      va_list args;
       gchar *string;
       
       va_start (args, format);
-      va_start (args2, format);
-      string = g_vsprintf ((gchar*) format, &args, &args2);
+      string = g_strdup_vprintf (format, args);
       va_end (args);
-      va_end (args2);
-      
-      string = g_strdup (string);
       
       scanner->msg_handler (scanner, string, FALSE);
       
@@ -636,10 +627,10 @@ g_scanner_cur_token (GScanner *scanner)
   return scanner->token;
 }
 
-GValue
+GTokenValue
 g_scanner_cur_value (GScanner *scanner)
 {
-  register GValue v;
+  GTokenValue v;
   
   v.v_int = 0;
   g_return_val_if_fail (scanner != NULL, v);
@@ -1060,7 +1051,7 @@ g_scanner_stat_mode (const gchar *filename)
 
 static void
 g_scanner_free_value (GTokenType     *token_p,
-		      GValue	     *value_p)
+		      GTokenValue     *value_p)
 {
   switch (*token_p)
     {
@@ -1082,7 +1073,7 @@ g_scanner_free_value (GTokenType     *token_p,
 static void
 g_scanner_get_token_i (GScanner	*scanner,
 		       GTokenType	*token_p,
-		       GValue		*value_p,
+		       GTokenValue	*value_p,
 		       guint		*line_p,
 		       guint		*position_p)
 {
@@ -1136,7 +1127,7 @@ g_scanner_get_token_i (GScanner	*scanner,
 static void
 g_scanner_get_token_ll	(GScanner	*scanner,
 			 GTokenType	*token_p,
-			 GValue		*value_p,
+			 GTokenValue	*value_p,
 			 guint		*line_p,
 			 guint		*position_p)
 {
@@ -1147,8 +1138,8 @@ g_scanner_get_token_ll	(GScanner	*scanner,
   register gboolean		in_string_dq;
   static   guchar		ch;
   register GTokenType		token;
-  register GValue		value;
   register GString		*gstring;
+  GTokenValue			value;
   
   config = scanner->config;
   (*value_p).v_int = 0;
