@@ -1,5 +1,5 @@
 /* GLIB - Library of useful routines for C programming
- * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald
+ * Copyright (C) 1995-1998  Peter Mattis, Spencer Kimball and Josh MacDonald
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,9 +21,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include <pwd.h>
 #include <sys/types.h>
 #include <sys/param.h>
+
+/* implement Glib's inline functions
+ */
+#define	G_INLINE_FUNC extern
+#define	G_CAN_INLINE 1
 #include "glib.h"
 
 const guint glib_major_version = GLIB_MAJOR_VERSION;
@@ -32,7 +38,41 @@ const guint glib_micro_version = GLIB_MICRO_VERSION;
 const guint glib_interface_age = GLIB_INTERFACE_AGE;
 const guint glib_binary_age = GLIB_BINARY_AGE;
 
-extern char* g_vsprintf (const gchar *fmt, va_list *args, va_list *args2);
+
+void
+g_atexit (GVoidFunc func)
+{
+  gint result;
+  gchar *error = NULL;
+
+  /* keep this in sync with glib.h */
+
+#ifdef	G_NATIVE_ATEXIT
+  result = ATEXIT (func);
+  if (result)
+    error = g_strerror (errno);
+#elif defined (HAVE_ATEXIT)
+#  ifdef NeXT /* @#%@! NeXTStep */
+  result = !atexit ((void (*)(void)) func);
+  if (result)
+    error = g_strerror (errno);
+#  else
+  result = atexit ((void (*)(void)) func);
+  if (result)
+    error = g_strerror (errno);
+#  endif /* NeXT */
+#elif defined (HAVE_ON_EXIT)
+  result = on_exit ((void (*)(int, void *)) func, NULL);
+  if (result)
+    error = g_strerror (errno);
+#else
+  result = 0;
+  error = "no implementation";
+#endif /* G_NATIVE_ATEXIT */
+
+  if (error)
+    g_error ("Could not register atexit() function: %s", error);
+}
 
 gint
 g_snprintf (gchar	*str,
@@ -40,7 +80,7 @@ g_snprintf (gchar	*str,
 	    gchar const *fmt,
 	    ...)
 {
-#ifdef HAVE_VSNPRINTF
+#ifdef	HAVE_VSNPRINTF
   va_list args;
   gint retval;
   
@@ -49,24 +89,46 @@ g_snprintf (gchar	*str,
   va_end (args);
   
   return retval;
-  
-#else
+#else	/* !HAVE_VSNPRINTF */
   gchar *printed;
-  va_list args, args2;
+  va_list args;
   
   va_start (args, fmt);
-  va_start (args2, fmt);
-  
-  printed = g_vsprintf (fmt, &args, &args2);
-  strncpy (str, printed, n);
-  str[n-1] = '\0';
-  
-  va_end (args2);
+  printed = g_strdup_vprintf (fmt, args);
   va_end (args);
   
-  return strlen (str);
+  strncpy (str, printed, n);
+  str[n-1] = '\0';
+
+  g_free (printed);
   
-#endif
+  return strlen (str);
+#endif	/* !HAVE_VSNPRINTF */
+}
+
+gint
+g_vsnprintf (gchar	 *str,
+	     gulong	  n,
+	     gchar const *fmt,
+	     va_list      args)
+{
+#ifdef	HAVE_VSNPRINTF
+  gint retval;
+  
+  retval = vsnprintf (str, n, fmt, args);
+  
+  return retval;
+#else	/* !HAVE_VSNPRINTF */
+  gchar *printed;
+  
+  printed = g_strdup_vprintf (fmt, args);
+  strncpy (str, printed, n);
+  str[n-1] = '\0';
+
+  g_free (printed);
+  
+  return strlen (str);
+#endif /* !HAVE_VSNPRINTF */
 }
 
 guint	     
